{% load i18n %}
{% load humanize %}

<div role="tabpanel" class="tab-pane" id="standings">
    <h3>{% translate 'NPC Standings' %}</h3>
    <div class="table-responsive">
        <table class="table table-striped table-width-fix" id="tab_standings">
            <thead>
                <tr>
                    <th>{% translate 'Standing' %}</th>
                    <th>{% translate 'Name' %}</th>
                    <th>{% translate 'NPC Type' %}</th>
                    <th>{% translate 'Eff. Standing (unmodifed)' %}</th>
                </tr>
            </thead>
            <tbody></tbody>
        </table>
    </div>
    {% if connection_skills_differ %}
        <p class="text-warning">
            Please note that modfied standings can not be relied on
            for criminal organizations (agents, NPC corporations, factions),
            with have positive base standings.
        </p>
    {% endif %}
    {% include 'memberaudit/partials/character_viewer/tabs/update_status.html' with update_section=sections_update_status.standings %}
</div>
