# Generated by Django 4.0.10 on 2023-12-19 16:03

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("memberaudit", "0017_characterasset_bigauto_id"),
    ]

    operations = [
        migrations.CreateModel(
            name="CharacterCloneInfo",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("last_clone_jump_date", models.DateTimeField(default=None, null=True)),
                (
                    "last_station_change_date",
                    models.DateTimeField(default=None, null=True),
                ),
                (
                    "character",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="clone_info",
                        to="memberaudit.character",
                    ),
                ),
                (
                    "home_location",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="memberaudit.location",
                    ),
                ),
            ],
            options={
                "default_permissions": (),
            },
        ),
    ]
