# Generated by Django 4.2.16 on 2024-10-07 19:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("memberaudit", "0019_alter_charactercontractitem_record_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="characterrole",
            name="role",
            field=models.CharField(
                choices=[
                    ("AT1", "account take 1"),
                    ("AT2", "account take 2"),
                    ("AT3", "account take 3"),
                    ("AT4", "account take 4"),
                    ("AT5", "account take 5"),
                    ("AT6", "account take 6"),
                    ("AT7", "account take 7"),
                    ("ACT", "accountant"),
                    ("AUD", "auditor"),
                    ("BMG", "brand manager"),
                    ("COM", "communications officer"),
                    ("CEQ", "config equipment"),
                    ("CSE", "config starbase equipment"),
                    ("CT1", "container take 1"),
                    ("CT2", "container take 2"),
                    ("CT3", "container take 3"),
                    ("CT4", "container take 4"),
                    ("CT5", "container take 5"),
                    ("CT6", "container take 6"),
                    ("CT7", "container take 7"),
                    ("CMG", "contract manager"),
                    ("DCT", "deliveries container take"),
                    ("DQY", "deliveries query"),
                    ("DTK", "deliveries take"),
                    ("DPL", "diplomat"),
                    ("DRT", "director"),
                    ("FCM", "factory manager"),
                    ("FTM", "fitting manager"),
                    ("HQ1", "hangar query 1"),
                    ("HQ2", "hangar query 2"),
                    ("HQ3", "hangar query 3"),
                    ("HQ4", "hangar query 4"),
                    ("HQ5", "hangar query 5"),
                    ("HQ6", "hangar query 6"),
                    ("HQ7", "hangar query 7"),
                    ("HT1", "hangar take 1"),
                    ("HT2", "hangar take 2"),
                    ("HT3", "hangar take 3"),
                    ("HT4", "hangar take 4"),
                    ("HT5", "hangar take 5"),
                    ("HT6", "hangar take 6"),
                    ("HT7", "hangar take 7"),
                    ("JAC", "junior accountant"),
                    ("PSM", "personnel manager"),
                    ("PMG", "project manager"),
                    ("RFF", "rent factory facility"),
                    ("RFC", "rent office"),
                    ("RRF", "rent research facility"),
                    ("SCO", "security officer"),
                    ("SPM", "skill plan manager"),
                    ("SDO", "starbase defense operator"),
                    ("SFT", "starbase fuel technician"),
                    ("STM", "station manager"),
                    ("TRD", "trader"),
                ],
                help_text="Role of a character",
                max_length=3,
            ),
        ),
    ]
