{% load i18n %}

<div role="tabpanel" class="tab-pane" id="skillqueue">
    <h3>{% translate 'Skill Queue' %}</h3>
    {% if is_training %}
        <div class="table-responsive">
            <table class="table table-striped table-width-fix" id="tab_skillqueue">
                <tbody></tbody>
            </table>
        </div>
    {% else %}
        <p class="text-warning">{% translate "Training not active" %}</p>
    {% endif %}

    {% include 'memberaudit/partials/character_viewer/tabs/update_status.html' with update_section=sections_update_status.skill_queue %}
</div>
